<?php
/*

    This file is part of Yet Another Nopaste (YaNoPaste!).

    YaNoPaste! is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with YaNoPaste!.  If not, see <http://www.gnu.org/licenses/>.

*/
class ParseResult {
    public $name = "";
    public $id = "";
    public $leaf = "";
    public $pwd = "";
    public $setPwd = true;
    public $original = "Error: original paste not found!";
    public $transform = "Error: transformed paste not found!";
    public $forDiff = false;
    public $diffLeaf = "";
    public $diffOriginal = "Error: original diff paste not found!";
    private $htmlTree = "";
    private $parsed = false;
    
    private function generateTreeLink($currentLeaf, $currentName, $nodeLink, $diffLink) {
        $base = generateFileURLFromId($this->id);
        $currentName = trim($currentName);
        if (strlen($currentName) == 0) {
            $currentName = "<i>noname</i>";
        }
        $this->htmlTree .= "<li>";
        if ($nodeLink) {
            $this->htmlTree .= "<a href=\"" 
            . $base . "&amp;leaf=" 
            . $currentLeaf
            . "\">" .$currentName . "</a> - ";
        } else {
            $this->htmlTree .= $currentName . " - ";
        }
        if ($diffLink) {
            $this->htmlTree .= "<a href=\""
            . $base . "&amp;diffLeaf=" 
            . $currentLeaf . "&amp;leaf=" 
            . $this->leaf
            . "\">[DIFF]</a>";
        } else {
            $this->htmlTree .= "[DIFF]";
        }
        $this->htmlTree .= "</li>";
    }
    
    private function parse() {
        if($this->parsed) {
            // already parsed
            return;
        }
        if($this->leaf == "") {
            $this->leaf = "leaf_" . $this->id;
        }
        $this->htmlTree = "Replies:<ul>";
        // Initializing XML Reader with given id
        $xmlreader = new XMLReader();
        $xmlreader->open(generateFileName($this->id . ".xml"));
        $inSelected = false;
        $inSelectedDiff = false;
        // First of all, we need to find the paste to print
        while($xmlreader->read()) {
            if($xmlreader->name == "paste") {
                if ($xmlreader->nodeType == XMLReader::ELEMENT) {
                    //Checking if this is the paste we are looking for...
                    if($xmlreader->getAttribute("leaf") == $this->leaf) {
                        $inSelected = true;
                        $inSelectedDiff = false;
                        $this->name = $xmlreader->getAttribute("name"); 
                        $this->generateTreeLink(
                            $xmlreader->getAttribute("leaf"), 
                            $xmlreader->getAttribute("name"),
                            true, false);
                    } else if($xmlreader->getAttribute("leaf") == $this->diffLeaf) {
                        $inSelected = false;
                        $inSelectedDiff = true;
                        $this->generateTreeLink(
                            $xmlreader->getAttribute("leaf"), 
                            $xmlreader->getAttribute("name"),
                            true, false);
                    } else {
                        $inSelected = false;
                        $inSelectedDiff = false;
                        $this->generateTreeLink(
                            $xmlreader->getAttribute("leaf"), 
                            $xmlreader->getAttribute("name"),
                            true, true);
                    }
                } else {
                    // nothing to do
                }
            } else if($xmlreader->name == "yanopaste" && $xmlreader->nodeType == XMLReader::ELEMENT) {
                $this->pwd = $xmlreader->getAttribute("pwd");
                $this->setPwd = ($xmlreader->getAttribute("setpwd") == "1");
            } else if($xmlreader->name == "original"
                      && $xmlreader->nodeType == XMLReader::ELEMENT) {
                if($inSelected) {
                    $this->original = getTextFromXMLReader($xmlreader);
                } else if($inSelectedDiff) {
                    $this->diffOriginal = getTextFromXMLReader($xmlreader);
                }
            } else if($xmlreader->name == "transform"
                      && $xmlreader->nodeType == XMLReader::ELEMENT) {
                if($inSelected) {
                    $this->transform = getTextFromXMLReader($xmlreader);
                } else if($inSelectedDiff) {
                    $this->diffTransform = getTextFromXMLReader($xmlreader);
                }
            } else if($xmlreader->name == "replies") {
                if ($xmlreader->nodeType == XMLReader::ELEMENT && !$xmlreader->isEmptyElement) {
                    $this->htmlTree .= "<ul>";
                } else if($xmlreader->nodeType == XMLReader::END_ELEMENT) {
                    $this->htmlTree .= "</ul>";
                } else {
                    // nothing to do
                }
            } else {
                // nothing to do
            }
        }
        $xmlreader->close();
        $this->htmlTree .= "</ul>";
        $this->parsed = true;
    }
    public function checkAccess() {
        $this->parse();
        if($this->setPwd) {
            $res = $_SESSION["access_" . $this->id];
            if(!$res) {
                $res = $this->pwd == $_POST["pwd"];
                $_SESSION["access_" . $this->id] = $res;
                return $res;
            } else {
                return true;
            }
        } else {
            // No password
            return true;
        }
    }
    public function printResultPage() {
        $this->parse();
        $printTransform = "";
        $printOriginal = "";
        if($this->forDiff) {
            // The content is XML escaped, which is fine when displaying the result in the HTML page. But here we need
            // to achive a comparison first. Therefore we need to unescape, diff, and escape again...
            $this->diffOriginal = unEscapeXML($this->diffOriginal);
            $this->original = unEscapeXML($this->original);
            $printOriginal = xdiff_string_diff($this->diffOriginal, $this->original);
            // Creating geshi object
            $geshi = new GeSHi($printOriginal, "diff");
            $geshi->enable_line_numbers(GESHI_FANCY_LINE_NUMBERS);
            //$geshi->enable_classes();
            $geshi->enable_strict_mode(true);
            $printTransform = $geshi->parse_code();
            $printOriginal = escapeXML($printOriginal);
        } else {
            $printTransform = $this->transform;
            $printOriginal = $this->original;
        }
        echo(getHTMLHeader(" - Yest Another NoPaste! - " . $this->name, ""));
        ?>
            <input type="button"
                   value="Initializing..."
                   disabled="disabled"
                   id="content_switcher"
                   onclick="switchPastedView()"/>
            <form name="reply_form"
                  method="POST"
                  action="index.php">
                <input type="submit" value="Reply" onclick="reply()"/>
                <input type="hidden" name="reply-to-id" value="<?=$this->id?>"/>
                <input type="hidden" name="reply-to-leaf" value="<?=$this->leaf?>"/>
            </form>
            <div id="tree"><?=$this->htmlTree?></div>
            <div id="highlight"><?=$printTransform?></div>
            <pre id="original"><?=$printOriginal?></pre>
        <?php
        echo(getHTMLFooter());
    }
}
?>
