<?php
/*

    This file is part of Yet Another Nopaste (YaNoPaste!).

    YaNoPaste! is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with YaNoPaste!.  If not, see <http://www.gnu.org/licenses/>.

*/
class ParseSave {
    public $reply = false;
    public $leaf = "";
    public $newLeaf = "";
    public $name = "";
    public $language = "";
    public $password = "";
    public $original = "";
    public $retention = 24;
    private $id = "";
    private $out = NULL;
    private $in = NULL;
    
    public function getId() {
        $this->init();
        return $this->id;
    }
    public function setId($pId) {
        $this->id = $pId;
    }
    public function save() {
        $this->init();
        $filename = generateFileName($this->id . ".xml");
        $this->out = new XMLWriter();
        $this->out->openURI($filename . ".tmp");
        $this->out->setIndent(true);
        $this->out->setIndentString("");        
        if(!$this->reply) {
            $this->newLeaf = "leaf_" . $this->id;
            $this->out->startElement("yanopaste");
            $this->out->writeAttribute("id", $this->id);
            $this->out->writeAttribute("retention", $this->retention);
            $this->out->writeAttribute("language", $this->language);
            $this->out->writeAttribute("setpwd", $this->password != "");
            $this->out->writeAttribute("pwd", $this->password);
            $this->out->writeAttribute("base-url", generateFileURL(""));
            $this->writePaste();
            $this->out->endElement();
        } else {
            $this->newLeaf = "leaf_" . $this->generateBaseId();
            $inParentPasteDepth = -10;
            $this->in = new XMLReader();
            $this->in->open($filename);
            while($this->in->read()) {
                switch($this->in->nodeType) {
                    case XMLReader::ELEMENT:
                        $this->out->startElement($this->in->name);
                        if($this->in->name == "yanopaste") {
                            $this->language = $this->in->getAttribute("language");
                        }
                        if($this->in->name == "paste" && $this->in->getAttribute("leaf") == $this->leaf) {
                            $inParentPasteDepth = $this->in->depth;
                        }
                        while($this->in->moveToNextAttribute()) {
                            $this->out->writeAttribute($this->in->name, $this->in->value);
                        }
                        if($this->in->isEmptyElement) {
	                        if($this->in->name == "replies" 
	                        && $inParentPasteDepth == ($this->in->depth - 1)) {
	                            $this->writePaste();
	                            $inParentPasteDepth = -10;
	                        }
                            $this->out->endElement();
                        }
                    break;
                    case XMLReader::END_ELEMENT:
                        if($this->in->name == "replies" && $inParentPasteDepth == ($this->in->depth - 1)) {
                            $this->writePaste();
                            $inParentPasteDepth = -10;
                        }
                        $this->out->endElement();
                    break;
                    case XMLReader::TEXT:
                    case XMLReader::WHITESPACE:
                        $this->out->text($this->in->value);
                    break;
                    default:
                    break;
                }
            }
        }

        
        $this->out->flush();
        $this->out->endDocument();
        if($this->reply) {
            $this->in->close();
        }
        rename($filename . ".tmp", $filename);
    }
    
    private function writePaste() {
        $this->out->startElement("paste");
        $this->out->writeAttribute("name", $this->name);
        $this->out->writeAttribute("leaf", $this->newLeaf);
        $this->out->startElement("original");
        $this->out->text(escapeXML($this->original));
        $this->out->endElement();
        $this->out->startElement("transform");
        $this->out->writeAttribute("engine", "geshi");
        $this->out->text($this->parseGeshi());
        $this->out->endElement();
        $this->out->startElement("replies");
        $this->out->endElement();
        $this->out->endElement();
    }
    
    private function parseGeshi() {
        // Creating geshi object
        $geshi = new GeSHi($this->original, $this->language);
        $geshi->enable_line_numbers(GESHI_FANCY_LINE_NUMBERS);
        //$geshi->enable_classes();
        $geshi->enable_strict_mode(true);
        return $geshi->parse_code();
    }
    
    private function init() {
        global $_YANOPASTE;
        if ($this->id != "") {
            return;
        }
        if ($this->retention != "forever") {
            $this->retention = $_YANOPASTE["lifetimes"][$this->retention];
        }
        $this->id = $this->generateBaseId() . "-" . $this->retention;
        $this->initialized = true;
    }
    
    private function generateBaseId() {
        $now = getdate();
        return $now["year"]
                 . str_pad($now["mon"], 2, "0", STR_PAD_LEFT)
                 . str_pad($now["mday"], 2, "0", STR_PAD_LEFT)
                 . str_pad($now["hours"], 2, "0", STR_PAD_LEFT)
                 . str_pad($now["minutes"], 2, "0", STR_PAD_LEFT)
                 . str_pad($now["seconds"], 2, "0", STR_PAD_LEFT)
                 . "_" . rand(10000,99999);
    }
}
?>
