<?php
/*

    This file is part of Yet Another Nopaste (YaNoPaste!).

    YaNoPaste! is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with YaNoPaste!.  If not, see <http://www.gnu.org/licenses/>.

*/

function initCookies() {
    $expire = time()+60*60*24*30;
    if(isset($_POST["language"])) {
        setCookie("YANOPASTE_language", $_POST["language"], $expire);
    }
    if(isset($_POST["name"])) {
        setCookie("YANOPASTE_name", $_POST["name"], $expire);
    }
    if(isset($_POST["setpwd"])) {
        setCookie("YANOPASTE_setpwd", $_POST["setpwd"], $expire);
    }
}

function generateFileName($id) {
    global $_YANOPASTE;
    $repository = "";
    if (!strrpos($id, "forever")) {
        $repository = $_YANOPASTE["repository"];
    } else {
        $repository = $_YANOPASTE["repository_forever"];
    }
    // ensuring the directory exist, otherwise trying to create it
    if (!file_exists($repository)) {
        mkdir($repository, 0755, true);
    }
    return $repository . "/paste" . $id;
}

function generateFileURL($path) {
    return "http://" . $_SERVER["SERVER_NAME"] . substr($_SERVER["PHP_SELF"], 0, strrpos($_SERVER["PHP_SELF"], "/")) . "/" . $path;
}

function generateFileURLFromId($id) {
    return "http://" . $_SERVER["SERVER_NAME"] . substr($_SERVER["PHP_SELF"], 0, strrpos($_SERVER["PHP_SELF"], "/")) . "/index.php?id=" . $id;
}

function retentionTimeAsString($time) {
    $days = floor($time / 24);
    $hours = $time % 24;
    $res = "";
    if ($days > 0) {
        $res .= $days . " day" . ($days > 1 ? "s " : " ");
    }
    if ($hours > 0) {
        $res .= $hours . " hour" . ($hours > 1 ? "s " : "");
    }
    return $res; 
}

function writeRetentionChoice() {
    global $_YANOPASTE;
    $trigger = false;
    $limit = $_YANOPASTE["lifetimes_requires_captcha"];
    foreach ($_YANOPASTE["lifetimes"] as $key => $time) {
        echo("<input type=\"radio\" value=\"" . 
            $key . "\" name=\"retention\" id=\"retention_" . 
            $key . "\"");
        if ($_YANOPASTE["lifetimes_default"] == $key) {
            echo(" checked=\"checked\"");
        }
        echo(" onclick=\"switchAnswer(" . ($limit > 0 && $time >= $limit) . ")\"");
        echo("/><label for=\"retention_" . $key . "\">" . retentionTimeAsString($time) . "</label>&nbsp;");
    }
    if ($_YANOPASTE["lifetimes_add_forever"]) {
        echo("<input type=\"radio\" value=\"forever\" name=\"retention\" id=\"retention_forever\"");
        echo(" onclick=\"switchAnswer(" . ($limit > 0) . ")\"");
        echo("<label for=\"retention_forever\">Forever</label>&nbsp;");
    }
    $_SESSION["answer"] = rand(1,25);
    $opened = ($_YANOPASTE["lifetimes"][$_YANOPASTE["lifetimes_default"]] >= 
              $_YANOPASTE["lifetimes_requires_captcha"]) ? "block" : "none";
    echo("<div id=\"div-answer\" style=\"display:" . $opened . 
         "\">If I have 50 euros and I buy " . $_SESSION["answer"] . 
         " candies, 2 euros each, I'll finnally have " .
         "<input name=\"answer\" value=\"\"/> in my pocket</div>");
}

function checkRetentionChoice() {
    global $_YANOPASTE;
    $limit = $_YANOPASTE["lifetimes_requires_captcha"];
    $choosen = $_POST["retention"];
    $check = $limit > 0 && ($choosen == "forever" || $_YANOPASTE["lifetimes"][$choosen] >= $limit);
    if ($check) {
        return $_POST["answer"] == (50 - 2 * $_SESSION["answer"]);
    } else {
        return true;
    }
}

function displayError($message) {
    echo(getHTMLHeader("Error!",""));
    echo($message);
    echo("<br/>Press the back button of your browser");
}

function getHTMLHeader($extTitle, $rootPath) {
    global $_YANOPASTE;
    if ($_YANOPASTE["ga_enabled"] == "true") {
        $ga_include = "<script src=\"http://www.google-analytics.com/ga.js\" type=\"text/javascript\"></script>";
    } else {
        $ga_include = "";
    }
    return "<html>" .
           "    <head>" .
           "        <title>Yet Another Nopaste " . $extTitle . "</title>" .
           "        <link rel=\"stylesheet\" type=\"text/css\" href=\"" . $rootPath . "screen.css\" media=\"screen\"/>" .
           "        <script type=\"text/javascript\" src=\"" . $rootPath . "common.js\"></script>" .
           "        " . $ga_include .
           "    </head>" .
           "    <body onload=\"init()\">" .
           "        <input type=\"hidden\" id=\"ga_enabled\" value=\"" . $_YANOPASTE["ga_enabled"] . "\"/>" .
           "        <input type=\"hidden\" id=\"ga_id\" value=\"" . $_YANOPASTE["ga_id"] . "\"/>" .
           "        <div id=\"header\" onclick=\"window.location.href='" . generateFileURL("") . "'\"></div>" .
           "        <div id=\"body\">";
}

function getHTMLFooter() {
    return "        </div>" .
           "        <div id=\"footer\">" .
           "            Yet Another Nopaste - version 1.3.0- GPL software - Sources available at " .
           "            <a href=\"http://sourceforge.net/projects/yanopaste\"><img id=\"sourceforge_logo\" src=\"http://sflogo.sourceforge.net/sflogo.php?group_id=237687&type=1\" alt=\"SourceForge.net Logo\" /></a>" .
           "            - <a href=\"http://yanopaste.sourceforge.net\">Demo</a>" .
           "            <br/> Copyright (C) 2010  <a href=\"http://www.jujunie.com\">Julien BETI</a>" .
           "        </div>" .
           "    </body>" .
           "</html>";
}

function cleanRepository() {
    $now = time()/3600;
    $path = "";
    global $_YANOPASTE;
    // Iterating over all files in the repository
    foreach (scandir($_YANOPASTE["repository"]) as $f) {
        $path = $_YANOPASTE["repository"] . "/" . $f;
        $retStart = strrpos($f, "-");
        $retEnd = strrpos($f, ".");
        $retention = substr($f, $retStart + 1, $retEnd - $retStart -1);
        // to-6765.xml
        // 0123456789
        //   2    7
        // 3, 7-2-1 = 4 
        if (is_file($path)) {
            if ($now - filemtime($path)/3600 > $retention) {
                unlink($path);
            } else {
                // File must be kept
            }
        } else {
            //nothing to do
        }
    }
}

function fileExists($id) {
    return file_exists(generateFileName($id) . ".xml");
}

function getDocument($id) {
    // Loading XML
    $xml = new DOMDocument;
    $xml->load(generateFileName($id) . ".xml");
    return $xml;
}

function getOriginalContent($xml, $leaf) {
    $xmlreader = new XMLReader();
    $xmlreader->open($xml . ".xml");
    $res = "Error: original content not found!";
    //Looking for the reply node
    while($xmlreader->read()) {
        if($xmlreader->name == "paste") {
            //Checking if this is the paste we are looking for...
            if($xmlreader->getAttribute("leaf") == $leaf) {
                // Found!. Now looking for the original node
                while($xmlreader->read()) {
                    if($xmlreader->name == "original") {
                        $res = $xmlreader->readString();
                        break;
                    }
                }
                break;
            }
        }
    }
    $xmlreader->close();
    return $res;
}

function getTextFromXMLReader($xmlreader) {
    while($xmlreader->read() && $xmlreader->nodeType != XMLReader::TEXT) {}
    return $xmlreader->value;
}

function escapeXML($toEscape) {
    $xmlFind    = array("&"    ,"<"    , ">"   , "\""    , "'");
    $xmlReplace = array("&amp;", "&lt;", "&gt;", "&quot;", "&apos;");
    return str_replace($xmlFind, $xmlReplace, $toEscape);
}

function unEscapeXML($toEscape) {
    $xmlReplace = array("<"   , ">"   , "\""    , "'"     , "&");
    $xmlFind    = array("&lt;", "&gt;", "&quot;", "&apos;", "&amp;");
    return str_replace($xmlFind, $xmlReplace, $toEscape);
}
?>
