<?php
/*

    This file is part of Yet Another Nopaste (YaNoPaste!).

    YaNoPaste! is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with YaNoPaste!.  If not, see <http://www.gnu.org/licenses/>.

*/
// Include config
require_once 'config.php';
// Include functions
include_once 'functions.php';
// Include geshi library
include_once 'includes/geshi.php';
// Include YaNoPaste! object classes definition
include_once 'ClassParseResult.php';
include_once 'ClassParseSave.php';

session_start();
initCookies();

if (isset($_GET["id"])) {
    // Found an ID => Parsing XML/XSL and output result
    $postedId = $_GET["id"];
    $idExist = fileExists($postedId);
    if (!$idExist) {
        // Perhaps it's a old id...
        $postedId .= "-" . $_YANOPASTE["upgrade"]["old_lifetime"];
        $idExist = fileExists($postedId);
    }
    if ($idExist) {
        $_SESSION["reply-to-id"] = $postedId;
        $result = new ParseResult();
        $result->id = $postedId;
        if (isset($_GET["leaf"])) {
            $result->leaf = $_GET["leaf"];
        }
        $result->forDiff = isset($_GET["diffLeaf"]);
        if($result->forDiff) {
            $result->diffLeaf = $_GET["diffLeaf"];
        }
    	if ($result->checkAccess()) {
            $result->printResultPage();
        } else {
            echo(getHTMLHeader(" - Please enter password!", ""));
            ?>
            <form name="frm" action="index.php?id=<?=$result->id?>&leaf=<?=$result->leaf?>" method="POST">
                <label for="pwd">The document you've requested is password protected. Please enter the p<u>a</u>ssword below:</label><br/>
                <?php
                if (isset($_POST["tryed"])) {
                    echo("<div class=\"error\">Password incorrect. Please retry</div>");
                }
                ?>
                <input type="password"
                        value=""
                        name="pwd"
                        id="pwd"
                        accesskey="a"/>
                <input type="hidden" name="tryed"/>
                <input type="submit" value="Validate" accesskey="V"/>
            </form>
            <?php
            echo(getHTMLFooter());
        }
    } else {
        echo(getHTMLHeader(" - Paste not found!", ""));
        echo("Sorry! Paste not found. Either the given URL is incorrect, either the paste you're requesting is no longer accessible");
        echo(getHTMLFooter());
    }
} else {
    if (isset($_POST["name"])) {
        if (checkRetentionChoice()) {
            $result = new ParseSave();
            $result->reply = isset($_POST["reply-to-leaf"]);
            if($result->reply) {
                $result->setId($_SESSION["reply-to-id"]);
                $result->leaf = $_POST["reply-to-leaf"];
            } else {
                $result->retention = $_POST["retention"];
                $result->password = $_POST["pwd"];
                $result->language = $_POST["language"];
            }
            $result->name = $_POST["name"];
            $result->original = stripslashes($_POST["content"]);
            $result->save();
            cleanRepository();
            header("Location: " . generateFileURLFromId($result->getId()) . "&leaf=" . $result->newLeaf);
        } else {
            displayError("Invalid answer to candies question!");
        }
    } else {
        $content = "";
        echo(getHTMLHeader(" - Paste into it!", ""));
        ?>
        <form name="frm" action="index.php" method="POST" onsubmit="return validateForm()">
            <fieldset>
                <legend>Paste</legend>
                <table>
                    <tr>
                        <th><label for="name"><u>N</u>ame</label></th>
                        <td>
                            <input type="text"
                                   value="<?=(isset($_COOKIE["YANOPASTE_name"]) ? $_COOKIE["YANOPASTE_name"] : "")?>"
                                   name="name"
                                   id="name"
                                   accesskey="N"/>
                        </td>
                        <?php
                        if (isset($_POST["reply-to-id"])) {
                            // we cannot trust the submitted id. Using the one in session
                            // as password has been checked...
                            $docPath = generateFileName($_SESSION["reply-to-id"]);
                            // Getting content for the paste
                            $content = getOriginalContent($docPath, $_POST["reply-to-leaf"]);
                            // setting the leaf id has hidden field for server processing.
                            // the id. will be retreived in session
                            ?>
                            </tr>
                            <tr>
                                <td colspan="4">
                                    Replying to paste...
                                    <input type="hidden" name="reply-to-leaf" value="<?=$_POST["reply-to-leaf"]?>"/>
                                </td>
                            <?php
                        } else {
                            ?>
                            <td>
                                <input type="checkbox"
                                       name="setpwd"
                                       id="setpwd"
                                       value="true"
                                       accesskey="S"
                                       <?=(isset($_COOKIE["YANOPASTE_setpwd"]) && $_COOKIE["YANOPASTE_setpwd"] == "true" ? "checked=\"checked\"" : "")?>/>
                            </td>
                            <th><label for="ispwd"><u>S</u>et password ?</label></th>
                            <th><label for="pwd">P<u>a</u>ssword</label></th>
                            <td>
                                <input type="password"
                                       value=""
                                       name="pwd"
                                       id="pwd"
                                       onchange="managePassword(this)"
                                       accesskey="a"/>
                            </td>
                        </tr>
                        <tr>
                            <th><label for="language"><u>L</u>anguage</label></th>
                            <td colspan="5">
                                <select name="language" id="language" accesskey="L">
                                    <?php
                                    foreach($_YANOPASTE["language"] as $l) {
                                        echo("<option value=\"" . $l . "\" " . (isset($_COOKIE["YANOPASTE_language"]) && $_COOKIE["YANOPASTE_language"] == $l ? "selected=\"selected\"" : "") . ">" . $_YANOPASTE["language"][$l] . "</option>");
                                    }
                                    ?>
                                </select>
                            </td>
                        </tr>
                        <tr>
                            <th><label for="language">Retention</label></th>
                            <td colspan="5">
                                <?php
                                writeRetentionChoice();
                                ?>
                            </td>
                        </tr>
                        <?php
                    }
                    ?>
                    <tr>
                        <th colspan="6"><label for="content"><u>C</u>ontent</label></th>
                    </tr>
                    <tr>
                        <td colspan="6"><textarea value="" name="content" id="content" accesskey="C"><?=$content?></textarea></td>
                    </tr>
                </table>
            </fieldset>
            <input type="submit" value="Paste!" accesskey="P"/>
        </form>
        <?php
    }
    echo(getHTMLFooter());
}
?>

