<?php
/*

    This file is part of Yet Another Nopaste (YaNoPaste!).

    YaNoPaste! is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    Foobar is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with YaNoPaste!.  If not, see <http://www.gnu.org/licenses/>.

*/
// Include config
require_once 'config.php';
// Include functions
include_once 'functions.php';
// Include geshi library
include_once 'includes/geshi.php';
echo(getHTMLHeader(" - Now upgrading...", ""));
?>
<ul>
  <li>Checking repository and upgrade files
  <?php
    // Iterating over all files in the repository
    $upgCounter = 0;
    foreach (scandir($_YANOPASTE["repository"]) as $f) {
        if(upgV1ToV2($f)) {
            $upgCounter++;
        }
    }
  ?>
  </li>
<?php
echo("Upgraded files: " . $upgCounter);
echo(getHTMLFooter());
// upgrade functions
function upgV1Tov2($f) {
    global $_YANOPASTE;
    if (strrpos($f, "paste") !== false && !strrpos($f, "-")) {
        // old file naming!. Needs to add retention...
        $retention = $_YANOPASTE["upgrade"]["old_lifetime"];
        $path = $_YANOPASTE["repository"] . "/" . $f;
        $newPath = substr($path, 0, strrpos($path, "."));
        $newPath .= "-" . $retention . ".xml";
        $in = new XMLReader();
        $out = new XMLWriter();
        $in->open($path);
        $language = "";
        $id = "";
        $original = "";
        $password = "";
        $name = "";
        while($in->read()) {
            switch($in->nodeType) {
                case XMLReader::ELEMENT:
                    if($in->name == "yanopaste") {
                        $language = $in->getAttribute("language");
                        $id = $in->getAttribute("id")."-".$retention;
                        $password = $in->getAttribute("password");
                        $name = $in->getAttribute("name");
                    }
                    if($in->name == "original") {
                        $original = getTextFromXMLReader($in);
                    }
                break;
                default:
                break;
            }
        }
        $out->openURI($newPath);
        $out->startElement("yanopaste");
        $out->writeAttribute("id", $id);
        $out->writeAttribute("retention", $retention);
        $out->writeAttribute("language", $language);
        $out->writeAttribute("setpwd", $password != "");
        $out->writeAttribute("pwd", $password);
        $out->writeAttribute("base-url", generateFileURL(""));
        $out->startElement("paste");
        $out->writeAttribute("name", $name);
        $out->writeAttribute("leaf", "leaf_" . $id);
        $out->startElement("original");
        //original is already XML escaped...
        $out->text($original);
        $out->endElement();
        $out->startElement("transform");
        $out->writeAttribute("engine", "geshi");
        // Creating geshi object
        $geshi = new GeSHi(unEscapeXML($original), $language);
        $geshi->enable_line_numbers(GESHI_FANCY_LINE_NUMBERS);
        //$geshi->enable_classes();
        $geshi->enable_strict_mode(true);
        $out->text($geshi->parse_code());
        $out->endElement();
        $out->startElement("replies");
        $out->endElement();
        $out->endElement();
        $out->endElement();
        $out->flush();
        $out->endDocument();
        $in->close();
        unlink($path);
        return true;
    } else {
        // Not a v1 file
        return false;
    }
}
?>
